--Command.System.Strict()

local addon, shared = ...

local data = {
	updated = false,
	events = {},
	guardian = { dwarf = true, highelf = true, mathosian = true }
}

-- ////////////////////////// Helpers /////////////////////////////////////////

-- Inspect.Unit.Detail("player").alliance may change temporarily (when player
-- is a mercenary). It appears safer to obtain the alliance by checking the
-- race.
local function getPlayerAlliance()
	local race = Inspect.Unit.Detail("player").race
	return data.guardian[race] and "guardian" or "defiant"
end

local function addEvent(item)
	data.events[item[6]] = {   -- achievementId
		plane = item[1],
		zone = item[2],
		name = item[3],
		boss = item[4],
		achievement = item[5],
		state = ""
	}
end

local function printEvent(event)
	print(string.format("*** %s (%s)\n- %s (%s)\n- %s (%s)",
		event.name, event.zone, event.boss, event.plane, event.achievement, event.state))
end

local function searchEvents(filterState, filterPlane, filterZone, filterPattern)
	local results = 0
	local plane = string.lower(filterPlane)
	local zone = string.lower(filterZone)
	local pattern = string.lower(filterPattern)
	for k, v in pairs(data.events) do
		local byState = not filterState or (v.state == shared.text.missing)
		local byPlane = (#plane == 0) or (string.find(string.lower(v.plane), plane) ~= nil)
		local byZone = (#zone == 0) or (string.find(string.lower(v.zone), zone) ~= nil)
		local eventText = string.lower(v.name .. " " .. v.boss .. " " .. v.achievement)
		local byPattern = (#pattern == 0) or (string.find(eventText, pattern) ~= nil)
		if byState and byPlane and byZone and byPattern then
			printEvent(v)
			results = results + 1
		end
	end
	if results == 0 then
		print("No matching events found.")
	end
end

local function getState(achievement, alliance)
	if achievement.alliance == nil or achievement.alliance == alliance then
		return (achievement.complete == true) and shared.text.complete or shared.text.missing
	else
		return (achievement.alliance == "guardian") and shared.text.guardian or shared.text.defiant
	end
end

local function updateAchievements()
	local playerAlliance = getPlayerAlliance()
	for k, v in pairs(data.events) do
		local detail = Inspect.Achievement.Detail(k)
		if detail ~= nil then
			v.achievement = detail.name   -- I hope this is the localized name..
			v.state = getState(detail, playerAlliance)
		else   -- initialisation error (wrong achievement id)!
			print("Error: No achievement details found for event " .. v.name)
		end
	end
end

-- ////////////////////////// Event handlers //////////////////////////////////

local function complete(handle, id)
	local event = data.events[id]
	if event ~= nil then
		event.state = shared.text.complete
	end
end

-- ////////////////////////// Slash command handlers //////////////////////////

local function lookup(handle, args)
	if #args == 0 then
		print(string.format(shared.text.usage, addon.toc.Version))
	else
		if not data.updated then
			updateAchievements()
			data.updated = true
		end
		local params = Library.LibGetOpt.getopt("mp:z:", args)
		searchEvents(params.m == true, params.p or "", params.z or "", params.leftover)
	end
end

-- ////////////////////////// Addon initialisation ////////////////////////////

for i, v in ipairs(shared.events) do
	addEvent(v)
end
shared.events = nil

Command.Event.Attach(Event.Achievement.Complete, complete, "Achievement.Complete callback")

Command.Event.Attach(Command.Slash.Register("elt"), lookup, "Slash command 'elt'")

--print("Initialized")
